unit peedit2;

interface

uses peedit, classes, controls, graphics, wintypes, messages, winprocs, stdctrls, dsgnintf,
     typinfo, forms;

type
  TVpe=class(TLabel)
  protected
    FOnActivate : TNotifyEvent;
    FPropertyEditor : TPropertyEditor;
    FVisualPropertyEditor : TVisualPropertyEditor;
    FOnEditSubProperties : TNotifyEvent;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); Override;
    Procedure FixBounds;
    Procedure SetPropertyEditor(Value : TPropertyEditor);
    Procedure SetOnEditSubProperties(Value : TNotifyEvent);
  public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
    Procedure UpdateCaption;
    Procedure DoActivate(Sender : TObject);
    Procedure Renew; virtual;                 { Causes visual value to be updated }
    Procedure Activate;
    Procedure Deactivating(Sender : TObject);
    Property PropertyEditor : TPropertyEditor Read FPropertyEditor Write SetPropertyEditor;
    Property OnEditSubProperties : TNotifyEvent Read FOnEditSubProperties Write SetOnEditSubProperties;
    Property OnActivate : TNotifyEvent Read FOnActivate Write FOnActivate;
  end;

implementation

Procedure TVpe.SetBounds;
begin
  Inherited Setbounds(ALeft,ATop,AWidth,AHeight);
  FixBounds;
end;

Procedure TVpe.FixBounds;
begin
  If Assigned(FVisualPropertyEditor) then
    FVisualPropertyEditor.Setbounds(Left,Top,Width,Height);
end;

Procedure TVpe.SetPropertyEditor;
begin
  FPropertyEditor:=Value;
  FPropertyEditor.Initialize;
  UpdateCaption;
  If assigned(FVisualPropertyEditor) then
    FVisualPropertyEditor.PropertyEditor:=Value;
  DoActivate(Self);{SAMFIX XXX}
end;

Procedure TVpe.SetOnEditSubProperties;
begin
  FOnEditSubProperties:=Value;
  If Assigned(FVisualPropertyEditor) then
    FVisualPropertyEditor.OnEditSubProperties:=Value;
end;

Constructor TVPe.Create;
begin
  Inherited Create(AOwner);
  OnClick:=DoActivate;
  Color:=clWhite;
  AutoSize:=False;
  SetBounds(Left,Top,121,24);
end;

Destructor TVpe.Destroy;
begin
  try
    If FVisualPropertyEditor<>nil then begin
      FVisualPropertyEditor.FinishedEdit(nil); { nil is a dummy anyway }
      Deactivating(FVisualPropertyEditor);
    end;
  finally
    Inherited Destroy;
  end;
end;

Procedure TVpe.DoActivate;
begin
  Activate;
end;

Procedure TVpe.Activate;
begin
  If Assigned(FOnActivate) then FOnActivate(Self);
  FPropertyEditor.Activate;
  FVisualPropertyEditor:=TVisualPropertyEditor.Create(Self);
  FVisualPropertyEditor.PropertyEditor:=FPropertyEditor;
  FVisualPropertyEditor.Slave:=true;
  FVisualPropertyEditor.OnLooseFocus:=Deactivating;
  FixBounds;
  FVisualPropertyEditor.Parent:=Parent;
{  FVisualPropertyEditor.SetFocus;{SAMFIX XXXX}
  FVisualPropertyEditor.ShowVisualPropertyEditor;
end;

Procedure TVpe.Deactivating;
begin
  if (FVisualPropertyEditor=Sender) then begin
    If FVisualPropertyEditor<>nil then begin
      GetParentForm(Self).DefocusControl(FVisualPropertyEditor,True);
{      RemoveComponent(FVisualPropertyEditor);}
      FVisualPropertyEditor.Free;
      FVisualPropertyEditor:=nil;
    end;
  end;
  UpdateCaption;
end;

Procedure TVpe.UpdateCaption;
begin
  { Only set the caption if we all agree }
  If FPropertyeditor.AllEqual then
    Caption:=NiceValue(FPropertyEditor,FPropertyEditor.Value)
  else
    Caption:='';
end;

Procedure TVpe.Renew;
begin
  UpdateCaption;
  If assigned(FVisualPropertyEditor) then
    FVisualPropertyEditor.Renew;
end;

end.

